Package: bettermc
Title: Enhanced Fork-Based Parallelization
Version: 1.2.1
Date: 2023-04-09
Authors@R: 
    c(person(given = "Andreas",
            family = "Kersting",
            role = c("aut", "cre", "cph"),
            email = "andreas.kersting@gfk.com"),
      person(given = "GfK SE",
             role = "cph"),
      person(given = "R Core team",
             role = "ctb",
             comment = "'etry()' and its print method borrow a lot from base R"))
Description: Drop-in replacement for 'parallel::mclapply()' adding e.g.
    tracebacks, crash dumps, retries, condition handling, improved seeding,
    progress bars and faster inter process communication. Some of the internal
    functions are also exported for other use: 'etry()' (extended try),
    'copy2shm()/allocate_from_shm()' (copy to and allocate from POSIX shared
    memory), 'char_map/map2char()' (split a character vector into its unique
    elements and a mapping on these) and various semaphore related functions.
Biarch: true
License: MIT + file LICENSE
URL: https://github.com/gfkse/bettermc
BugReports: https://github.com/gfkse/bettermc/issues
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: checkmate, parallel
RoxygenNote: 7.2.3
Suggests: progress, spelling, testthat (>= 2.1.0)
Language: en-US
NeedsCompilation: yes
Packaged: 2023-04-09 19:04:20 UTC; andreas
Author: Andreas Kersting [aut, cre, cph],
  GfK SE [cph],
  R Core team [ctb] ('etry()' and its print method borrow a lot from base
    R)
Maintainer: Andreas Kersting <andreas.kersting@gfk.com>
Repository: CRAN
Date/Publication: 2023-04-12 16:20:12 UTC
Built: R 4.4.0; x86_64-apple-darwin18.7.0; 2024-05-08 23:02:14 UTC; unix
