# Generated by roxygen2: do not edit by hand

S3method(print,"etry-error")
export(allocate_from_shm)
export(char_map)
export(compress_chars)
export(copy2shm)
export(crash_dumps)
export(etry)
export(map2char)
export(mclapply)
export(sem_close)
export(sem_open)
export(sem_post)
export(sem_unlink)
export(sem_wait)
export(semv_open)
export(semv_post)
export(semv_unlink)
export(semv_wait)
export(uncompress_chars)
importFrom(utils,capture.output)
importFrom(utils,limitedLabels)
useDynLib(bettermc, .registration = TRUE, .fixes = "C_")
